from plone.i18n.normalizer import IIDNormalizer
from zope.component import getUtility
import inspect
import os
import re


def find_object_in_stack(name, klass):
    frame = inspect.currentframe()
    while not isinstance(frame.f_locals.get(name, None), klass):
        frame = frame.f_back
    return frame.f_locals[name]


def get_mimetype_css_class_from_icon_path(icon_path):
    normalizer = getUtility(IIDNormalizer)
    image_filename = os.path.basename(icon_path)
    image_filename = re.sub(r'\.png$', r'', image_filename)
    image_filename = re.sub(r'\.gif$', r'', image_filename)
    return 'icon-mimetype-img-{}'.format(
        normalizer.normalize(image_filename, max_length=255)
    )



from ftw.theming import utils
from plone.app.layout.icons import icons
from plone.memoize.instance import memoize


WRAPPER_TEMPLATE = '<span class="{classes}">{content}</span>'


class CatalogBrainContentIcon(icons.CatalogBrainContentIcon):

    @memoize
    def html_tag(self):
        image_tag = super(CatalogBrainContentIcon, self).html_tag()
        if image_tag is None:
            return None
        return WRAPPER_TEMPLATE.format(classes=self.wrapper_classes(),
                                       content=image_tag)

    def wrapper_classes(self):
        return 'mimetype-icon {}'.format(
            utils.get_mimetype_css_class_from_icon_path(self.url)
        )



from ftw.theming import utils
from operator import attrgetter
from operator import methodcaller
from Products.CMFCore.utils import getToolByName
from Products.CMFDynamicViewFTI.interfaces import IDynamicViewTypeInformation
from Products.Five import BrowserView


class ThemingIcons(BrowserView):

    def get_portal_types(self):
        portal_types = getToolByName(self.context, 'portal_types')
        return map(methodcaller('getId'),
                   filter(IDynamicViewTypeInformation.providedBy,
                          portal_types.objectValues()))

    def get_mime_types(self):
        mimetypes_registry = getToolByName(self.context, 'mimetypes_registry')
        icon_paths = sorted(set(map(attrgetter('icon_path'),
                                    mimetypes_registry.mimetypes())))

        def itemize(filename):
            name = utils.get_mimetype_css_class_from_icon_path(filename)
            return {
                'filename': filename,
                'classes': 'mimetype-icon {}'.format(name),
                'normalized_name': name.replace('icon-mimetype-img-', '')}

        return map(itemize, icon_paths)
